@@ -1,6 +1,6 @@
 /* Determine a canonical name for the current locale's character encoding.
 
-   Copyright (C) 2000-2006, 2008-2009 Free Software Foundation, Inc.
+   Copyright (C) 2000-2006, 2008-2013 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License along
-   with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 /* Written by Bruno Haible <bruno@clisp.org>.  */
 
@@ -30,11 +29,11 @@
 #include <stdlib.h>
 
 #if defined __APPLE__ && defined __MACH__ && HAVE_LANGINFO_CODESET
-# define DARWIN7 /* Darwin 7 or newer, i.e. MacOS X 10.3 or newer */
+# define DARWIN7 /* Darwin 7 or newer, i.e. Mac OS X 10.3 or newer */
 #endif
 
 #if defined _WIN32 || defined __WIN32__
-# define WIN32_NATIVE
+# define WINDOWS_NATIVE
 #endif
 
 #if defined __EMX__
@@ -44,7 +43,7 @@
 # endif
 #endif
 
-#if !defined WIN32_NATIVE
+#if !defined WINDOWS_NATIVE
 # include <unistd.h>
 # if HAVE_LANGINFO_CODESET
 #  include <langinfo.h>
@@ -57,7 +56,7 @@
 #  define WIN32_LEAN_AND_MEAN
 #  include <windows.h>
 # endif
-#elif defined WIN32_NATIVE
+#elif defined WINDOWS_NATIVE
 # define WIN32_LEAN_AND_MEAN
 # include <windows.h>
 #endif
@@ -83,7 +82,7 @@
 #endif
 
 #if defined _WIN32 || defined __WIN32__ || defined __CYGWIN__ || defined __EMX__ || defined __DJGPP__
-  /* Win32, Cygwin, OS/2, DOS */
+  /* Native Windows, Cygwin, OS/2, DOS */
 # define ISSLASH(C) ((C) == '/' || (C) == '\\')
 #endif
 
@@ -109,21 +108,33 @@
 #if __STDC__ != 1
 # define volatile /* empty */
 #endif
-/* Pointer to the contents of the charset.alias file, if it has already been
-   read, else NULL.  Its format is:
-   ALIAS_1 '\0' CANONICAL_1 '\0' ... ALIAS_n '\0' CANONICAL_n '\0' '\0'  */
-static const char * volatile charset_aliases;
+/* Pointer to the ordered array of charset aliases read.
+   Format of a single alias is: ALIAS '\0' CANONICAL '\0' */
+static const char ** volatile charset_aliases;
+/* Aliases count, -1 until charset aliases are read. */
+static int volatile charset_aliases_count = -1;
+/* fallback charset name, "*" entry from aliases */
+static const char * volatile charset_fallback;
 
-/* Return a pointer to the contents of the charset.alias file.  */
-static const char *
+static int
+charset_alias_cmp(const void *a,const void *b)
+{
+  return strcmp (*(const char**)a, *(const char**)b);
+}
+
+/* Reads the contents of the charset.alias file.  */
+static void
 get_charset_aliases (void)
 {
   const char *cp;
+  const char **cps = 0;
+  int cnt = 0;
+  const char *fb = 0;
+  const char *aliases;
 
-  cp = charset_aliases;
-  if (cp == NULL)
+  if (charset_aliases_count < 0)
     {
-#if !(defined DARWIN7 || defined VMS || defined WIN32_NATIVE || defined __CYGWIN__)
+#if !(defined DARWIN7 || defined VMS || defined WINDOWS_NATIVE || defined __CYGWIN__)
       const char *dir;
       const char *base = "charset.alias";
       char *file_name;
@@ -228,8 +239,7 @@ get_charset_aliases (void)
                         {
                           /* Out of memory. */
                           res_size = 0;
-                          if (old_res_ptr != NULL)
-                            free (old_res_ptr);
+                          free (old_res_ptr);
                           break;
                         }
                       strcpy (res_ptr + res_size - (l2 + 1) - (l1 + 1), buf1);
@@ -309,7 +319,7 @@ get_charset_aliases (void)
            "DECKOREAN" "\0" "EUC-KR" "\0";
 # endif
 
-# if defined WIN32_NATIVE || defined __CYGWIN__
+# if defined WINDOWS_NATIVE || defined __CYGWIN__
       /* To avoid the troubles of installing a separate file in the same
          directory as the DLL and of retrieving the DLL's directory at
          runtime, simply inline the aliases here.  */
@@ -340,10 +350,67 @@ get_charset_aliases (void)
 # endif
 #endif
 
-      charset_aliases = cp;
-    }
+      /* build ordered array of aliases and also find fallback */
+      cnt = 0;
+      for (aliases = cp; *aliases != '\0';
+	   aliases += strlen (aliases) + 1, aliases += strlen (aliases) + 1)
+	{
+	  if (aliases[0] == '*' && aliases[1] == '\0')
+	    fb = aliases + strlen (aliases) + 1;
+	  else
+	    cnt ++;
+	}
 
-  return cp;
+      if (cnt > 0)
+	{
+          cps = (const char **) malloc (cnt * sizeof(*cps));
+	  if (cps == NULL)
+	    {
+	      /* out of memory */
+	      charset_aliases_count = 0;
+	      return;
+	    }
+    
+          cnt = 0;
+          for (aliases = cp; *aliases != '\0';
+               aliases += strlen (aliases) + 1, aliases += strlen (aliases) + 1)
+	    {
+              if (!(aliases[0] == '*' && aliases[1] == '\0'))
+		cps[cnt++] = aliases;
+	    }
+    
+          qsort (cps, cnt, sizeof(*cps), charset_alias_cmp);
+	}
+
+      charset_aliases = cps;
+      charset_fallback = fb;
+      charset_aliases_count = cnt;
+    }
+}
+
+/* Replace codeset name using the alias table.
+   The result must not be freed; it is statically allocated.
+*/
+
+static const char *
+resolve_alias (const char *codeset)
+{
+  const char **alias_ptr = 0;
+
+  get_charset_aliases ();
+  
+  if (charset_aliases_count > 0)
+    alias_ptr = bsearch (&codeset, charset_aliases,
+                        charset_aliases_count, sizeof(codeset),
+                        charset_alias_cmp);
+  
+  if (alias_ptr != NULL)
+    return *alias_ptr + strlen (*alias_ptr) + 1;
+
+  if (charset_fallback != NULL)
+    return charset_fallback;
+
+  return codeset;
 }
 
 /* Determine the current locale's character encoding, and canonicalize it
@@ -359,9 +426,8 @@ const char *
 locale_charset (void)
 {
   const char *codeset;
-  const char *aliases;
 
-#if !(defined WIN32_NATIVE || defined OS2)
+#if !(defined WINDOWS_NATIVE || defined OS2)
 
 # if HAVE_LANGINFO_CODESET
 
@@ -369,10 +435,9 @@ locale_charset (void)
   codeset = nl_langinfo (CODESET);
 
 #  ifdef __CYGWIN__
-  /* Cygwin 1.5.x does not have locales.  nl_langinfo (CODESET) always
-     returns "US-ASCII".  As long as this is not fixed, return the suffix
-     of the locale name from the environment variables (if present) or
-     the codepage as a number.  */
+  /* Cygwin < 1.7 does not have locales.  nl_langinfo (CODESET) always
+     returns "US-ASCII".  Return the suffix of the locale name from the
+     environment variables (if present) or the codepage as a number.  */
   if (codeset != NULL && strcmp (codeset, "US-ASCII") == 0)
     {
       const char *locale;
@@ -409,10 +474,10 @@ locale_charset (void)
             }
         }
 
-      /* Woe32 has a function returning the locale's codepage as a number:
-         GetACP().  This encoding is used by Cygwin, unless the user has set
-         the environment variable CYGWIN=codepage:oem (which very few people
-         do).
+      /* The Windows API has a function returning the locale's codepage as a
+         number: GetACP().  This encoding is used by Cygwin, unless the user
+         has set the environment variable CYGWIN=codepage:oem (which very few
+         people do).
          Output directed to console windows needs to be converted (to
          GetOEMCP() if the console is using a raster font, or to
          GetConsoleOutputCP() if it is using a TrueType font).  Cygwin does
@@ -455,12 +520,12 @@ locale_charset (void)
 
 # endif
 
-#elif defined WIN32_NATIVE
+#elif defined WINDOWS_NATIVE
 
   static char buf[2 + 10 + 1];
 
-  /* Woe32 has a function returning the locale's codepage as a number:
-     GetACP().
+  /* The Windows API has a function returning the locale's codepage as a
+     number: GetACP().
      When the output goes to a console window, it needs to be provided in
      GetOEMCP() encoding if the console is using a raster font, or in
      GetConsoleOutputCP() encoding if it is using a TrueType font.
@@ -528,16 +593,7 @@ locale_charset (void)
     /* The canonical name cannot be determined.  */
     codeset = "";
 
-  /* Resolve alias. */
-  for (aliases = get_charset_aliases ();
-       *aliases != '\0';
-       aliases += strlen (aliases) + 1, aliases += strlen (aliases) + 1)
-    if (strcmp (codeset, aliases) == 0
-        || (aliases[0] == '*' && aliases[1] == '\0'))
-      {
-        codeset = aliases + strlen (aliases) + 1;
-        break;
-      }
+  codeset = resolve_alias (codeset);
 
   /* Don't return an empty string.  GNU libc and GNU libiconv interpret
      the empty string as denoting "the locale's character encoding",
@@ -545,5 +601,12 @@ locale_charset (void)
   if (codeset[0] == '\0')
     codeset = "ASCII";
 
+#ifdef DARWIN7
+  /* Mac OS X sets MB_CUR_MAX to 1 when LC_ALL=C, and "UTF-8"
+     (the default codeset) does not work when MB_CUR_MAX is 1.  */
+  if (strcmp (codeset, "UTF-8") == 0 && MB_CUR_MAX <= 1)
+    codeset = "ASCII";
+#endif
+
   return codeset;
 }
