@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-2004 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1993-2011 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: edit.h,v 1.35 2009/06/22 08:50:08 lav Exp $ */
-
 #ifndef EDIT_H
 #define EDIT_H
 
@@ -106,8 +104,8 @@ extern  char    *HOME,*TERM,*DISPLAY;
 
 extern  char    *buffer;
 extern  int     modified;
-extern  num     BufferSize;
-extern  num     GapSize;
+extern  offs    BufferSize;
+extern  offs    GapSize;
 extern  offs    ptr1,ptr2,oldptr1,oldptr2;
 extern  num     stdcol;
 
@@ -125,6 +123,8 @@ void SetEolStr(const char *);
 
 extern   int   TabsInMargin;
 
+extern char Program[];
+
 #define MemStep     (0x2000)
 #define Tabulate(c) ((( ((c)<0) ? ((c)-TabSize+1) : (c))/TabSize+1)*TabSize)
 
@@ -216,11 +216,12 @@ void        Edit();
 
 int         LockFile(int fd,bool drop_write_lock);
 int         CheckMode(mode_t);
-int	    file_check(char *);	 /* checks existence or ability to create */
+int	    file_check(const char *);	 /* checks existence or ability to create */
 
 void    DeleteToEOL();
 void    DrawFrames();
 void    ExpandAllTabs();
+void	ExpandSpanTabs();
 void    Options();
 void    ReadConf();
 void    editcalc();
@@ -297,6 +298,8 @@ void  ConvertFromDosToUnix(offs start,num size);
 
 int   Suffix(const char *,const char *);
 
+bool BlockEqAt(offs,const char *,int);
+
 #define STATUS_LINE_ATTR    find_attr(STATUS_LINE)
 #define NORMAL_TEXT_ATTR    find_attr(NORMAL_TEXT)
 #define BLOCK_TEXT_ATTR     find_attr(BLOCK_TEXT)
