@@ -22,12 +22,7 @@
 #include <stdlib.h>
 
 extern "C" {
-#ifdef WITH_REGEX
-   #include <regex.h>
-#else
-   #include <rx.h>
-   #define RE_TRANSLATE_TYPE unsigned char *
-#endif
+#include <regex.h>
 }
 
 extern int hl_option,hl_active,hl_lines;
@@ -43,11 +38,14 @@ struct syntax_hl
    re_registers regs;
 
    syntax_hl *next;
+   syntax_hl *sub;
 
+   static char *selector;
    static syntax_hl *chain;
-   static void free_chain();
+   static void free_chain(syntax_hl*);
    static void attrib_line(const char *buf1,int len1,const char *buf2,int len2,
 			   unsigned char *line);
+   static void make_els(const char *buf1,int len1,const char *buf2,int len2,int pos,int ll,syntax_hl *c,class element **els);
 
    syntax_hl(int color,int mask);
    ~syntax_hl();
