@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-2008 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1993-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: kern.cc,v 1.33 2008/09/08 11:50:08 lav Exp $ */
-
 #include <config.h>
 #include   <stdio.h>
 #include   <ctype.h>
@@ -339,8 +337,8 @@ int   InsertBlock(const char *block_left,num size_left,const char *block_right,n
 
    num   i;
    offs  oldoffset;
-   num   num_of_lines,num_of_columns,oldline,oldcol;
-   num	 num_of_lines_curr,num_of_columns_curr;
+   num   num_of_lines,num_of_columns,oldline;
+   num	 num_of_lines_curr;
    int   break_at;
    int   join_at;
    num	 size;
@@ -389,7 +387,7 @@ int   InsertBlock(const char *block_left,num size_left,const char *block_right,n
       }
    }
    num_of_lines=0;
-   oldcol=num_of_columns=GetCol();
+   num_of_columns=GetCol();
    oldline=GetLine();
    ptr1+=size_left;
    ptr2-=size_right;
@@ -397,7 +395,6 @@ int   InsertBlock(const char *block_left,num size_left,const char *block_right,n
 
    CalculateLineCol(&num_of_lines,&num_of_columns,oldoffset,oldoffset+size_left);
    num_of_lines_curr=num_of_lines;
-   num_of_columns_curr=num_of_columns;
    CalculateLineCol(&num_of_lines,&num_of_columns,oldoffset+size_left,oldoffset+size);
 
    join_at=-1;
@@ -973,8 +970,8 @@ offs   NextLine(offs ptr)
 	 ptr=ptr1;
 	 break;
       }
-      ptr=found-buffer;
-      if(BolAt(++ptr))
+      ptr=found+1-buffer;
+      if(BolAt(ptr))
 	 return ptr;
    }
    offs size=Size();
@@ -983,8 +980,8 @@ offs   NextLine(offs ptr)
       found=(const char*)memchr(buffer+ptr+GapSize,eol,size-ptr);
       if(!found)
 	 return size;
-      ptr=found-buffer-GapSize;
-      if(BolAt(++ptr))
+      ptr=found+1-buffer-GapSize;
+      if(BolAt(ptr))
 	 return ptr;
    }
    /*NOTREACHED*/
@@ -1289,3 +1286,16 @@ void SetEolStr(const char *n)
    EolSize=strlen(n);
    memcpy(EolStr,n,EolSize);
 }
+
+bool BlockEqAt(offs o,const char *s,int len)
+{
+   if(o+len>Size() || o<0)
+      return false;
+   if(o>=ptr1)
+      return !memcmp(buffer+GapSize+o,s,len);
+   if(o+len<=ptr1)
+      return !memcmp(buffer+o,s,len);
+   int len1=ptr1-o;
+   return !memcmp(buffer+o,s,len1)
+       && !memcmp(buffer+GapSize,s+len1,len-len1);
+}
