@@ -151,6 +151,7 @@ const ActionNameRec ActionNameTable[]=
    {CALCULATOR,"calculator"},
    {DRAW_FRAMES,"draw-frames"},
    {TABS_EXPAND,"expand-tabs"},
+   {SPAN_TABS_EXPAND,"expand-tab-spans"},
    {TEXT_OPTIMIZE,"optimize-text"},
    {CHOOSE_CHAR,"choose-character"},
    {CHOOSE_WCHAR,"choose-wide-character"},
@@ -813,7 +814,6 @@ int   GetNextAction()
 {
    unsigned char *store;
    int   key;
-   bool	 seen_func_key=false;
 
    store=StringTyped;
    StringTypedLen=0;
@@ -875,9 +875,7 @@ int   GetNextAction()
 	 }
 #endif
 
-	 if(key>UCHAR_MAX)
-	    seen_func_key=true;
-	 else
+	 if(key<=UCHAR_MAX)
 	 {
 	    *(store++)=key;
 	    *store=0;
