@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-2003 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1993-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: loadsave.cc,v 1.26 2009/06/22 08:27:23 lav Exp $ */
-
 #include <config.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -145,7 +143,7 @@ struct  menu   ConCan4Menu[]={
 off_t  GetDevSize(int fd)
 {
 #ifdef BLKGETSIZE
-   unsigned long sect=0;
+   unsigned sect=0;
    if(ioctl(fd,BLKGETSIZE,&sect)==0)
       return ((off_t)sect)<<9;
 #endif
@@ -217,9 +215,35 @@ int   LoadFile(char *name)
 
    newfile=0;
 
-   if(stat(name,&st)==-1 && errno==ENOENT)
+   const char *open_name=name;
+   unsigned long long mmap_begin=0;
+   unsigned long mmap_len=0;
+   char open_name1[256];
+   unsigned n;
+   if(buffer_mmapped) {
+      if(sscanf(name,"%[^:]:%lli:%li%n",open_name1,&mmap_begin,&mmap_len,&n)==3
+      && n==strlen(name))
+	 open_name=open_name1;
+      else
+	 mmap_begin=mmap_len=0;
+   }
+
+   if(stat(open_name,&st)!=-1)
    {
-      int f=creat(name,0644);
+      FileMode=st.st_mode;
+      if((!buffer_mmapped && (S_ISBLK(FileMode) || S_ISCHR(FileMode)))
+	 || S_ISFIFO(FileMode))
+      {
+	 ErrMsg("This is a special file or a pipe\nthat I cannot edit.");
+	 EmptyText();
+	 return(ERR);
+      }
+      if(S_ISDIR(FileMode))
+	 View|=2;
+   }
+   else if(errno==ENOENT && !View && !buffer_mmapped)
+   {
+      int f=creat(open_name,0644);
       if(f!=-1)
       {
 	 close(f);
@@ -234,31 +258,21 @@ int   LoadFile(char *name)
 	 return(ERR);
       }
    }
-   else if(errno==0)
-   {
-      FileMode=st.st_mode;
-      if((!buffer_mmapped && (S_ISBLK(FileMode) || S_ISCHR(FileMode)))
-	 || S_ISFIFO(FileMode))
-      {
-	 ErrMsg("This is a special file or a pipe\nthat I cannot edit.");
-	 EmptyText();
-	 return(ERR);
-      }
-      if(S_ISDIR(FileMode))
-	 View|=2;
-   }
-   file=open(name,(View?O_RDONLY:O_RDWR|O_CREAT),0644);
-   if(file==-1)
+   int open_flags=View?O_RDONLY:O_RDWR;
+   if(!View && !buffer_mmapped)
+	open_flags|=O_CREAT;
+   file=open(open_name,open_flags,0664);
+   if(file==-1 && !View)
    {
       View|=2;
-      file=open(name,O_RDONLY);
+      file=open(open_name,O_RDONLY);
 	 /* try to open the file in read-only mode */
-      if(file==-1)
-      {
-	 FError(name);
-	 EmptyText();
-	 return(ERR);
-      }
+   }
+   if(file==-1)
+   {
+      FError(open_name);
+      EmptyText();
+      return(ERR);
    }
 
    // re-stat the file in case it was created
@@ -329,15 +343,25 @@ int   LoadFile(char *name)
       }
       if(st.st_size>0)
       {
-	 buffer=(char*)mmap(0,st.st_size,PROT_READ|(View?0:PROT_WRITE),
-			    MAP_SHARED,file,0);
+	 if(mmap_len==0) {
+	    mmap_len=st.st_size;
+	    if((size_t)mmap_len!=st.st_size) {
+	       errno=ENOMEM;
+	       FError(name);
+	       EmptyText();
+	       return ERR;
+	    }
+	 }
+	 buffer=(char*)mmap(0,mmap_len,PROT_READ|(View?0:PROT_WRITE),
+			    MAP_SHARED,file,mmap_begin);
 	 if(buffer==(char*)MAP_FAILED)
 	 {
+	    buffer=0;
 	    FError(name);
 	    EmptyText();
 	    return ERR;
 	 }
-	 BufferSize=st.st_size;
+	 BufferSize=mmap_len;
 	 ptr1=ptr2=BufferSize;
 	 GapSize=0;
 	 TextEnd=TextPoint(BufferSize);
@@ -479,11 +503,14 @@ static int CreateBak(char *name)
 
    if(fd==-1)
    {
+      if(bfd!=-1)
+         close(bfd);
       FError(name);
       return ERR;
    }
    else if(bfd==-1)
    {
+      close(fd);
       FError(bakname);
       return ERR;
    }
@@ -508,6 +535,7 @@ static int CreateBak(char *name)
          }
          if(bytesread==0)
             break;
+	 // FIXME: partial writes
          if(write(bfd,buf2,bytesread)==-1)
 	 {
 	    FError(bakname);
