@@ -16,8 +16,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: screen.cc,v 1.49 2009/05/18 07:45:53 lav Exp $ */
-
 #include <config.h>
 #include <string.h>
 #include <ctype.h>
@@ -96,11 +94,9 @@ void  TestPosition()
       return;
    }
    num oldtop=ScreenTop.Line();
-   num newtop;
    if(GetLine()-oldtop>TextWinHeight-m-1)
    {
       ScreenTop=PrevNLines(Offset(),TextWinHeight-m-Scroll);
-      newtop=ScreenTop.Line();
       flag=REDISPLAY_ALL;
    }
    else if(GetLine()-ScreenTop.Line()<0)
@@ -296,7 +292,7 @@ void  MessageSync(const char *s)
    refresh();
 }
 
-void  ClearMessage()
+static void ClearMessageOnTextWin()
 {
    if(message_sp>0 && (ShowStatusLine!=SHOW_BOTTOM || message_sp>1))
    {
@@ -308,6 +304,13 @@ void  ClearMessage()
               TextWinHeight);
       ScrollBar(false);
    }
+}
+
+void  ClearMessage()
+{
+   ClearMessageOnTextWin();
+   if(message_sp>0 && ShowStatusLine==SHOW_BOTTOM)
+      StatusLine();
    message_sp=0;
 }
 
@@ -322,7 +325,8 @@ void  StatusLine()
    int   l;
    char  flags[16];
 
-   ClearMessage();
+   ClearMessageOnTextWin();
+   message_sp=0;
 
    if(ShowStatusLine==SHOW_NONE)
       return;
@@ -859,7 +863,7 @@ void  Redisplay(num line,offs ptr,num limit)
 		     if(MBCharWidth==0 && clwp>clw)
 		     {
 			int a=0;
-			while(clwp[-1].chars[a] && a<CCHARW_MAX)
+			while(a<CCHARW_MAX && clwp[-1].chars[a])
 			   a++;
 			if(a<CCHARW_MAX)
 			   clwp[-1].chars[a]=ch;
