@@ -1,6 +1,6 @@
-/* 
+/*
  * Copyright (c) 1993-1997 by Alexander V. Lukyanov (lav@yars.free.net)
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -30,3 +30,6 @@ void     ContReplace(void);
 void     StartSearchBackward(void);
 
 extern   History  SearchHistory;
+
+extern unsigned char map_to_lower[256];
+void map_to_lower_init();
