@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1993-2006 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1993-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: user.cc,v 1.56 2008/10/24 13:37:41 lav Exp $ */
-
 #include <config.h>
 #include <ctype.h>
 #ifdef HAVE_UNISTD_H
@@ -28,6 +26,7 @@
 #include <grp.h>
 #include <time.h>
 #include <string.h>
+#include <alloca.h>
 #include "edit.h"
 #include "block.h"
 #include "keymap.h"
@@ -429,6 +428,36 @@ void  UserPageTop()
 	 CurrentPos=ScreenTop;
    }
 }
+void UserScrollUp()
+{
+   if(hex) {
+      ScreenTop-=16;
+      if((CurrentPos-ScreenTop)/16>=TextWinHeight)
+	 CurrentPos-=16;
+   } else {
+      ScreenTop=PrevLine(ScreenTop);
+      if(GetLine()>=ScreenTop.Line()+TextWinHeight)
+	 UserLineUp();
+   }
+   flag=REDISPLAY_ALL;
+}
+void UserScrollDown()
+{
+   if(hex) {
+      if((TextEnd-ScreenTop)/16>=TextWinHeight) {
+	 ScreenTop+=16;
+	 if(CurrentPos<ScreenTop)
+	    CurrentPos+=16;
+      }
+   } else {
+      if(TextEnd.Line()-ScreenTop.Line()>=TextWinHeight) {
+	 ScreenTop=NextLine(ScreenTop);
+	 if(CurrentPos<ScreenTop)
+	    UserLineDown();
+      }
+   }
+   flag=REDISPLAY_ALL;
+}
 void  UserPageUp()
 {
    if(PreferPageTop)
@@ -568,7 +597,7 @@ void  UserCommentLine()
    || Suffix(FileName,".cxx")
    || Suffix(FileName,".java"))
    {
-      if(Char()=='/' && CharRel(1)=='/')
+      if(BlockEq("//",2))
       {
 	 DeleteBlock(0,2);
 	 if(Char()==' ')
@@ -581,7 +610,7 @@ void  UserCommentLine()
    }
    else if(Suffix(FileName,".sql"))
    {
-      if(Char()=='-' && CharRel(1)=='-')
+      if(BlockEq("--",2))
       {
 	 DeleteBlock(0,2);
 	 if(Char()==' ')
@@ -592,22 +621,23 @@ void  UserCommentLine()
 	 InsertBlock("-- ",3);
       }
    }
-   else if(Suffix(FileName,".c") || Suffix(FileName,".h"))
+   else if(Suffix(FileName,".c") || Suffix(FileName,".h")
+   || Suffix(FileName,".css"))
    {
-      if(Char()=='/' && CharRel(1)=='/')
+      if(BlockEq("//",2))
       {
 	 DeleteBlock(0,2);
 	 if(Char()==' ')
 	    DeleteBlock(0,1);
 	 goto done;
       }
-      if(Char()=='/' && CharRel(1)=='*')
+      if(BlockEq("/*",2))
       {
 	 unc=1;
 	 DeleteBlock(0,2);
       }
       ToLineEnd();
-      if(CharRel(-1)=='/' && CharRel(-2)=='*')
+      if(BlockEqLeft("*/",2))
       {
 	 unc=1;
 	 DeleteBlock(2,0);
@@ -619,6 +649,27 @@ void  UserCommentLine()
 	 InsertBlock("/*",2);
       }
    }
+   else if(Suffix(FileName,".html") || Suffix(FileName,".htm")
+   || Suffix(FileName,".shtml"))
+   {
+      if(BlockEq("<!--",4))
+      {
+	 unc=1;
+	 DeleteBlock(0,4);
+      }
+      ToLineEnd();
+      if(BlockEqLeft("-->",3))
+      {
+	 unc=1;
+	 DeleteBlock(3,0);
+      }
+      if(!unc)
+      {
+	 InsertBlock("-->",3);
+	 ToLineBegin();
+	 InsertBlock("<!--",4);
+      }
+   }
    else // default
    {
       if(Char()=='#')
@@ -895,12 +946,19 @@ int   UserSave()
       return(UserSaveAs());
 }
 
-int   file_check(char *fn)
+int   file_check(const char *fn)
 {
    char	 dir[256];
    char	 *slash;
    char	 msg[1024];
 
+   if(buffer_mmapped) {
+      char *open_name1=(char*)alloca(strlen(fn)+1);
+      long mmap_begin=0,mmap_len=0;
+      if(sscanf(fn,"%[^:]:%li:%li",open_name1,&mmap_begin,&mmap_len)==3)
+	 fn=open_name1;
+   }
+
    if(access(fn,R_OK)==-1)
    {
       if(access(fn,F_OK)==0)
@@ -909,7 +967,7 @@ int   file_check(char *fn)
 	 ErrMsg(msg);
 	 return ERR;
       }
-      if(View&1)  // view mode
+      if((View&1) || buffer_mmapped)  // view mode or mmap mode
       {
 	 sprintf(msg,"File: %s\nThe specified file does not exist",fn);
 	 ErrMsg(msg);
